% DESCRIPTION
% -----------
% Creates an array of partition matrices for N objects in 8 bins

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% DEPENDENCY
% Requires function Q.m - a repeated step in this program.

% OUTPUT - Part, a matrix which contains pages of the partition matrices
% required by convolution.m

% USER VARIABLE NUMBER OF PHOTONS
% -------------------------------
N = 12; % Max number of incoming photons to consider
% -------------------------------

inc = 0;
i = 1;
j = 1;
q = 0;
colflag = 0;

clear Part;
Part = zeros(1,8,N); % Put in first row of partition matrices with 8 bins


% ---- Number of photons loop ----
for inc = 1:N
    
    i = 1;
    j = 1;
    q = 0;
    colflag = 1;
    
    % First entry is always the number of incident photons
    Part(1,1,inc) = inc;
    
    % ---- Main Matrix Loop ----
    A=1;
    while A
        
        % ---- Entry Creating Loop ----
        B=1;
        while B
            j = j + 1;
            q = Q(sum((Part(i,1:(j-1),inc))),inc,Part(i,(j-1),inc));
            
            if q == 0
                B = 0;
            else
                Part(i,j,inc) = q;
            end
            
            % Leave loop if we have reached the final column
            if j == 8
                B = 0;
                
                % Finish if we've created a BAD row (which cannot sum to 
                % photon number), then make that a zero row
                if (sum(Part(i,1:8,inc)) < inc)
                    % If we made a bad row by reducing first element, zero
                    % it and then we're done
                    if colflag == 1 
                        A = 0;
                        Part(i,:,inc) = [0,0,0,0,0,0,0,0];
                    else
                        % Zero bad row and select previous row
                        Part(i,:,inc) = [0,0,0,0,0,0,0,0];
                        j = colflag;
                        i = i - 1;
                    end
                end
                
            end

        end % ---- End of Entry Creating Loop ----
        
        
        % ---- Reducing Entry Loop ----
        C = 1;
        while (C && A)
            j = j - 1;
            
            % Is the element greater than the one to its right?
            if ((Part(i,j,inc) > Part(i,j+1,inc)) && (Part(i,j,inc) > 1))
                i = i + 1;
                
                if j ~= 1
                    Part(i,1:(j-1),inc) = Part(i-1,1:(j-1),inc);
                    Part(i,j,inc) = Part(i-1,j,inc) - 1;
                else
                    Part(i,j,inc) = Part(i-1,j,inc) - 1;
                end

                colflag = j;
                C = 0;
            
            else
                
                if j==1 % If first element is not greater than the one to its right, we're done
                    A=0;
                    colflag = 1;
                end

            end
                   
        end % ---- End of Reducing Entry Loop ----
                
        
    end % ---- End of Main Matrix Loop ----


end % End of Number of Photons Loop