function rho_better = removebg(rho, bg1, bg2)
% Creator: Andy Worsley

% Finds a more ideal input state by taking out some Poissonian background
% from a state rho.

% I have not found a least squares algorithm for an equation of the form
% P = Q[rho]R, where all the variables are matrices, so the first part of 
% the program reduces the problem to a standard p = Mx form for a least 
% squares fit, where the components of x are the diagonals of [rho].

% Input:
% rho - a state with background


% -------------------------------------------------
%bg1 = 0.075; % Mean Poissonian background in LOOPY1
%bg2 = 0.045; % Mean Poissonian background in LOOPY2
% -------------------------------------------------

% Output:
% rho_better - a state with reduced background

% NOTE: I have not found a way to determine when the best state has been
% found. Making bg1 and bg2 bigger makes a better and better diagonal
% state.



% START

% ---- INITIALISE AND PREALLOCATE VARIABLES ----
N = 8;

n = 0; % Counting Variables
m = 0;
prow = 0;
qrow = 1;
qcol = 1;
rrow = 1;
rcol = 1;
mcol = 1;
mrow = 1;
xrow = 0;
loopnum = 0;



Q = zeros(81,N+1); 
R = zeros(81,N+1);

p = zeros(81,1);
M = zeros(81,N+1);

flag = 0;

% Import backgroundy state
P = rho;

% Create columns of vectors for storing test results
% resnorm = zeros(1,numlosses^2);

x_bestrn = zeros((N+1)^2,1);
rho_better = zeros(N+1,N+1);

% ---- END OF INITIALISATION ----

% Deconstruct P to make p, an 81x1 vector
prow = 1;
for n = 1:9
    for m = 1:9
        p(prow,1) = P(n,m);
        prow = prow + 1;
    end
end

% Create background matrices
B1 = Bground(bg1, N);
B2 = Bground(bg2, N);
        
% Calculate working matrices - initially need P = Q*rho*R
Q = B1;
R = B2';       
        
% Create 81x(N+1) M matrix - to convert to form p = Mx
qrow = 1;
qcol = 1;
rrow = 1;
rcol = 1;
mcol = 1;

mrow = 1;
for qrow = 1:(N+1)
    for rrow = 1:(N+1)
        mcol = 1;
        for qcol = 1:(N+1)
            for rcol = 1:(N+1)
                M(mrow,mcol) = Q(qrow,qcol)*R(rcol,rrow);
            mcol = mcol + 1;
            end
        end
    mrow = mrow + 1;
    end
end


% Solve least squares problem to find photon statistics

[x_bestrn(:,1)] = lsqlin(M,p,[],[],[],[],zeros(81,1),ones(81,1),[],optimset('TolFun', 1e-25, 'Display', 'Off'));
%[x_bestrn(:,1)] = lsqr(M,p,10e-25,10000);
%[x_bestrn(:,1)] = lsqnonneg(M,p);


% Use 81 x_bestrn elements to reconstruct input state rho, 9x9 matrix
xrow = 1;
for n = 1:(N+1)
    for m = 1:(N+1)
        rho_better(n,m) = x_bestrn(xrow,1);
        xrow = xrow + 1;   
    end
end

% Renormalise
rho_better = rho_better/sum(sum(rho_better));