function [ output ] = thermdist( mean, maxnum )
% THERMDIST - Creates a thermal distribution of photon numbers of size 
% 'maxnum'+1 with a mean 'mean', for photon numbers 0 to 'maxnum'

% Creator: Andy Worsley
% Dept. of Physics, Univeristy of Oxford
% March 2008

% INPUTS
% mean - the mean of the distribution
% maxnum - the maximum number of photons to consider

% OUTPUT
% rho - a matrix of the joint number state (diagonal)

rho = zeros(maxnum+1,maxnum+1);

for n=1:(maxnum+1)
    rho(n,n) = mean^(n-1)/(mean+1)^n;
end

output = rho;
    


